package edu.sdsc.inca;

import edu.sdsc.inca.util.ConfigProperties;
import edu.sdsc.inca.util.Constants;
import edu.sdsc.inca.util.ResourcesWrapper;
import edu.sdsc.inca.consumer.IncaConfigCache;
import edu.sdsc.inca.consumer.SuiteCache;
import edu.sdsc.inca.protocol.Protocol;
import org.mortbay.http.HttpContext;
import org.mortbay.http.HttpListener;
import org.mortbay.xml.XmlConfiguration;

import java.util.Properties;
import java.util.regex.Pattern;
import java.net.URL;
import java.io.File;

/**
 * Starts up a Jetty server to server our JSP pages. Also starts threads
 * to periodically load and cache the resources (from agent) and suites (from
 * depot).
 *
 * @author Shava Smallen &lt;ssmallen@sdsc.edu&gt;
 */
public class Consumer extends Component {
  // Constants
  final public static Pattern IMMEDIATE_PATTERN = Pattern.compile(
    Protocol.IMMEDIATE_SUITE_NAME
  );
  final private String JETTY_CONFIG = "jetty.xml";
  final private String JETTY_DIR = "jetty";

  // Static variables
  private static Properties clientConfig = null;
  private static int cacheMaxWaitPeriod = Constants.MILLIS_TO_MINUTE;
  private static int cacheReloadPeriod = 5 * Constants.MILLIS_TO_MINUTE;
  private static IncaConfigCache icCache = new IncaConfigCache();
  private static SuiteCache suiteCache = new SuiteCache();

  // Member variables
  org.mortbay.jetty.Server server = null;
  private String tempPath = null;

  // Configuration options
  public static final String CONSUMER_OPTS = ConfigProperties.mergeValidOptions(
    Component.COMPONENT_OPTS,
    "a|agent       str  URI to the Inca agent\n" +
    "d|depot       str  URI to the Inca depot\n" +
    "m|maxWait     int  Max wait time a JSP tag should wait on a cached item\n"+
    "r|reload      int  Reload period for cached objects (e.g., suites)\n" +
    "v|var         path Path to temporary directory\n",
    true
  );


  /**
   * Returns configuration properties for clients (e.g., agent and depot).
   *
   * @return a list of configuration properties for clients
   */
  public static Properties getClientConfiguration() {
    return clientConfig;
  }

  /**
   * Return the resources with a maximum wait time.
   *
   * @param maxWait The maximum time in milliseconds to wait for the resources
   *                to be initially loaded into the cache.
   *
   * @return  The resources as a ResourceWrapper or null if the maxWait is
   * exceeded
   */
  public static String[] getRepositories( int maxWait ) {
    return icCache.getRepositories( maxWait );
  }

  /**
   * Return the specified cached suite document with a maximum wait time.
   *
   * @param name   The name of the suite to fetch from the configured depot.
   * @param maxWait The maximum time in milliseconds to wait for the specified
   *                suite to be initially loaded into the cache.
   *
   * @return  An array of report summaries or null if the maxWait is exceeded
   */
  public static String[] getSuite( String name, int maxWait ) {
    return suiteCache.getSuite( name, maxWait );
  }

  /**
   * Return the resources with a maximum wait time.
   *
   * @param maxWait The maximum time in milliseconds to wait for the resources
   *                to be initially loaded into the cache.
   *
   * @return  The resources as a ResourceWrapper or null if the maxWait is
   * exceeded
   */
  public static String getSuiteConfigs( int maxWait ) {
    return icCache.getSuites( maxWait );
  }

  /**
   * Returns the path where the server stores temporary files.
   *
   * @return the temp directory path
   */
  public String getTempPath() {
    return this.tempPath;
  }

  /**
   * Return the resources with a maximum wait time.
   *
   * @param maxWait The maximum time in milliseconds to wait for the resources
   *                to be initially loaded into the cache.
   *
   * @return  The resources as a ResourceWrapper or null if the maxWait is
   * exceeded
   */
  public static ResourcesWrapper getResourceConfig( int maxWait ) {
    return icCache.getResourceConfig( maxWait );
  }

  /**
   * Return the amount of time (milliseconds) a JSP tag should wait for a suite
   * or resources before timing out.
   *
   * @return  the number of milliseconds a JSP tag should wait for a cached
   * object to return.
   */
  public static int getCacheMaxWaitPeriod() {
    return cacheMaxWaitPeriod;
  }

  /**
   * Return the period of refresh for the suite and resources in the
   * Consumer's cache.
   *
   * @return  the number of milliseconds in between reloads of cached
   * objects.
   */
  public static int getCacheReloadPeriod() {
    return cacheReloadPeriod;
  }

  public synchronized boolean isRunning() {
    return this.server != null && this.server.isStarted();
  }

  /**
   * Runs a Jetty server.
   */
  public void startConsumer() throws Exception  {

    // get jetty config file
    URL jettyUrl = ClassLoader.getSystemClassLoader().getResource(JETTY_CONFIG);
    if(jettyUrl == null) {
      logger.error( JETTY_CONFIG + " not found in classpath" );
      return;
    }
    XmlConfiguration jettyConfig =  new XmlConfiguration( jettyUrl );

    // configure server; also set temp dir
    this.server = new org.mortbay.jetty.Server();
    jettyConfig.configure( this.server );
    HttpContext[] contexts = this.server.getContexts();
    for ( int i = 0; i < contexts.length; i++ ) {
      contexts[i].setTempDirectory
        ( new File(this.getTempPath() + File.separator + JETTY_DIR + i ) );

    }

    // print out some info to the logs
    HttpListener[] listeners = this.server.getListeners();
    for ( int i = 0; i < listeners.length; i++ ) {
      logger.info
        ( "Starting Jetty " + listeners[i].getClass().getName() + " server: " +
          listeners[i].getDefaultScheme() + "://localhost:" +
          listeners[i].getPort() );
    }

    // start the server and the cache preloads
    logger.debug("Starting jetty server");
    this.server.start();
    logger.debug("Jetty server started");
    icCache.start();
    suiteCache.start();
  }

  public synchronized void shutdown() {
    if ( server.isStarted() ) {
      try {
        server.stop(); // JJH Is this safe?  Why not interrupt()?
      } catch ( InterruptedException e1 ) {
        logger.error( "Shutdown process interrupted", e1 );
      }
    }
    icCache.interrupt();
    suiteCache.interrupt();
    logger.info( "Shutdown complete" );
  }

  /**
   * Read in the configuration for the consumer from the specified properties
   * list.
   *
   * @param config  A list of configuration options for the consumer
   *
   * @throws ConfigurationException
   */
  public void setConfiguration(Properties config) throws ConfigurationException{
    super.setConfiguration( config );
    String prop;

    // check that agent and depot are specified and save the configuratioin
    if((prop = config.getProperty("agent")) == null) {
      throw new ConfigurationException( "No agent URI specified" );
    }
    if((prop = config.getProperty("depot")) == null) {
      throw new ConfigurationException( "No depot URI specified" );
    }
    config.remove( "logfile" ); // so log does not get initialized in client
    Consumer.setClientConfiguration( config );

    // configure consumer
    if((prop = config.getProperty("maxWait")) != null) {
      Consumer.setCacheMaxWaitPeriod(
        (new Integer(prop)).intValue() * Constants.MILLIS_TO_SECOND
      );
    }
    if((prop = config.getProperty("reload")) != null) {
      Consumer.setCacheReloadPeriod(
        (new Integer(prop)).intValue() * Constants.MILLIS_TO_SECOND
      );
    }
    if((prop = config.getProperty("var")) != null) {
      this.setTempPath(prop);
    }
  }

  /**
   * Set the amount of time (milliseconds) a JSP tag should wait for a suite
   * or resources before timing out.
   *
   * @param period the number of milliseconds a JSP tag should wait for a cached
   * object to return.
   */
  public static void setCacheMaxWaitPeriod( int period ) {
    Consumer.cacheMaxWaitPeriod = period;
  }

  /**
   * Set the period of refresh for the suite and resources in the
   * Consumer's cache.
   *
   * @param period the number of milliseconds in between reloads of cached
   * objects
   */
  public static void setCacheReloadPeriod( int period ) {
    Consumer.cacheReloadPeriod = period;
  }

  /**
   * Sets the configuration properties for clients (e.g., agent and depot).
   *
   * @param props   a list of configuration properties for clients
   */
  public static void setClientConfiguration( Properties props ) {
    logger.debug( "Setting client configuration" );
    clientConfig = props;
  }

  /**
   * Sets the directory path where the Server stores temporary files.
   *
   * @param path the temporary directory path
   * @throws ConfigurationException if the path can't be read from the classpath
   */
  public void setTempPath(String path) throws ConfigurationException {
    File tempPath = new File(path);
    if (!tempPath.exists() && !tempPath.mkdir()) {
      throw new ConfigurationException
        ("Unable to create temporary dir '" + path + "'");
    }
    this.tempPath = tempPath.getAbsolutePath();
    logger.info("Placing temporary files in " + this.tempPath);
  }

  /**
   * Start up the consumer server.
   *
   * @param args  array of command line arguments to configure the consumer.
   */
  public static void main(final String[] args) {

    Consumer consumer = new Consumer();
    try {
      configComponent(
        consumer, args, CONSUMER_OPTS, "inca.consumer.", "edu.sdsc.inca.Consumer",
        "inca-consumers-version"
      );
      consumer.startConsumer();
      while(consumer.isRunning()) {
        Thread.sleep(2000);
      }
    } catch(Exception e) {
      logger.fatal("Configuration error: ", e);
      System.err.println("Configuration error: " + e);
      System.exit(1);
    }
  }
}
